package PSoC_Programmer1;
use Time::HiRes qw(usleep gettimeofday);
use strict;
use warnings;
use IPC::Open3;
use threads;
use Thread::Queue;

BEGIN {
  use Exporter   ();
  our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

  # set the version for version checking
  $VERSION     = 1.00;
  # if using RCS/CVS, this may be preferred
  #  $VERSION = sprintf "%d.%03d", q$Revision: #1 $ =~ /(\d+)/g;

  @ISA         = qw(Exporter);
#                    &ReadProgram

  @EXPORT      = qw(&start
                    &getfuntime
                    &getretstatus
                    &getrettime
                    &getports
                    &OpenPort
                    &GetProgrammerVersion
                    &runcommand
                    &doread
                    &quit

                    &Acquire
                    &Calibrate
                    &AcquireAndCal
                    &CheckSum
                    &Erase
                    &EraseBlock
                    &GetSiliconId
                    &PowerOff
                    &PowerOn
                    &Program
                    &ProgramBlock
                    &Protect
                    &ReadBlock
                    &ReadProm
					&PSoC3_DebugPortConfig
					&PSoC3_ProgramRow
					&PSoC3_ReadRow

                    &SetAcquireMode
                    &SetChipType
                    &SetPowerVoltage
                    &SetSiliconId
                    &Verify
                    &SetHexFile
                    &ReadHexData
                    &ReadHexChecksum
                    &WriteHexData
                    &WriteHexFile
                   );
  %EXPORT_TAGS = ( );           # eg: TAG => [ qw!name1 name2! ],

  # your exported package globals go here,
  # as well as any optionally exported functions
  @EXPORT_OK   = qw();
}
;

sub new {
  my $self = {};
  bless ($self);
  return $self;
}
;
sub a {
  my $xref = shift;
  my @data = (1, 2, 3, 4, 5);
  @${xref} = @data;
}

my @xx = (4 ,5);
a(\@xx);
my $WRFH;
my $RDFH: shared=1;
my $ERFH;
my $DataQueue;
my $thr;
my $out;
my @start_time;
my @end_time;
my $delta_time;
sub start {
  my $self = shift;
  @start_time = gettimeofday();
  $self->{command} = [@_];
  $self->{args} = shift;
  my $pid = IPC::Open3::open3($WRFH,$RDFH,$ERFH,@{$self->{command}});
  $self->{pid} = $pid;
  $DataQueue = Thread::Queue->new;
  $thr = threads->new (\&doreadthread);
  $self->{timeout} = 10;        # 1 seconds
  # start it
  my $o = $self->runcommand("help\n\n\n");
  $self->{help} = $o;
  $self->doread();
  usleep(100);
  @end_time = gettimeofday();calc_delta();
  return $pid;
};
sub DESTROY {
  my $self = shift;
  if(defined($WRFH) && defined($RDFH)){
  $self->quit();
  }
}
sub doreadthread {
  while (<$RDFH>) {
    s/\cM//g;
    $DataQueue->enqueue($_);
  }
}
sub getports {
  my $self = shift;
  @start_time = gettimeofday();
  my $o = $self->runcommand("GetPorts");
  my @ports = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return @ports;
}
sub OpenPort {
  my $self = shift;
  @start_time = gettimeofday();
  my $portname = shift;
  my $directory = shift;
  my $x = "OpenPort " . $portname . " " . $directory;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub DAP_Acquire {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = "DAP_Acquire";
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}  
sub PSoC3_GetTemp {
  my $self = shift;
  @start_time = gettimeofday();
  my $sampleNumber = shift;
  my $timerPeriod = shift;
  my $clockDivider = shift;
  my $x = "PSoC3_GetTemp " . $sampleNumber . " " . $timerPeriod . " " . $clockDivider;
  my $o = $self->runcommand($x);
  my @val = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return @val;
}
sub PSoC3_CheckSum {
  my $self = shift;
  my $arrayID = shift;
  my $startRowID = shift;
  my $rowsPerArray = shift;
  @start_time = gettimeofday();
  my $x = "PSoC3_CheckSum ".$arrayID." ".$startRowID." ".$rowsPerArray;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub PSoC3_ReadHexImageSizes {
  my $self = shift;
  @start_time = gettimeofday();
  my $o = $self->runcommand("PSoC3_ReadHexImageSizes");
  my @val = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return @val;
}
sub PSoC3_DebugPortConfig {
  my $self = shift;
  my $bytes = shift;
  @start_time = gettimeofday();
  my $o = $self->runcommand("PSoC3_DebugPortConfig ".$bytes);
  my @val = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return @val;
}
sub ReadHexFile {
  my $self = shift;
  my $param = shift;
  @start_time = gettimeofday();
  my $cmd = "ReadHexFile"." ".$param;
  my $o = $self->runcommand($cmd);
  my @val = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return @val;
}
sub JTAG_EnumerateDevices {
  my $self = shift;
  @start_time = gettimeofday();
  my $o = $self->runcommand("JTAG_EnumerateDevices");
  my @val = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return @val;
}
sub jtagio {
  my $self = shift;
  @start_time = gettimeofday();
  my $read = shift;
  my $tdi_tms = shift;
  my $x = "jtagio " . $read . " " . $tdi_tms;
  my $o = $self->runcommand($x);
  #my @val = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub jtagiow {
  my $self = shift;
  @start_time = gettimeofday();
  my $Addr = shift;
  my $data = shift;
  my $x = "jtagiow " . $Addr . " " . $data;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub jtagior {
  my $self = shift;
  @start_time = gettimeofday();
  my $Addr = shift;
  my $x = "jtagior " . $Addr;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub swdiow {
  my $self = shift;
  @start_time = gettimeofday();
  my $Addr = shift;
  my $data = shift;
  my $x = "swdiow " . $Addr . " " . $data;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub swdior {
  my $self = shift;
  @start_time = gettimeofday();
  my $Addr = shift;
  my $x = "swdior " . $Addr;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub swdiow_raw {
  my $self = shift;
  @start_time = gettimeofday();
  my $data = shift;
  my $x = "swdiow_raw " . $data;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub swdior_raw {
  my $self = shift;
  @start_time = gettimeofday();
  my $data = shift;
  my $x = "swdior_raw " . $data;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub SPI_ConfigureBus {
  my $self = shift;
  @start_time = gettimeofday();
  my $bitOrder = shift;
  my $mode = shift;
  my $freq = shift;
  my $extra = shift;
  my $x = "SPI_ConfigureBus " . $bitOrder . " " . $mode . " " . $freq . " " . $extra;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub SPI_GetSupportedFreq {
  my $self = shift;
  my $x = "SPI_GetSupportedFreq";
  my $o = $self->runcommand($x);
  my @arr = split(' ',$o);
  @end_time = gettimeofday();calc_delta();
  return @arr;
}
sub SPI_DataTransfer {
  my $self = shift;
  my $mode = shift;
  my $dataIN = shift;
  my $x = "SPI_DataTransfer " . $mode . " " . $dataIN;
  my $o = $self->runcommand($x);
  my @arr = split(' ',$o);
  @end_time = gettimeofday();calc_delta();
  return @arr;
}
sub PSoC3_GetFlashArrayInfo {
  my $self = shift;
  my $ArrayID = shift;
  my $x = "PSoC3_GetFlashArrayInfo " . $ArrayID;
  my $o = $self->runcommand($x);
  my @arr = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return @arr;
}
sub PSoC3_GetEccStatus {
  my $self = shift;
  my $ArrayID = shift;
  my $o = $self->runcommand("PSoC3_GetEccStatus");
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub PSoC3_GetEepromArrayInfo {
  my $self = shift;
  my $ArrayID = shift;
  my $x = "PSoC3_GetEepromArrayInfo " . $ArrayID;
  my $o = $self->runcommand($x);
  my @arr = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return @arr;
}
sub PSoC3_ProgramRowFromHex {
  my $self = shift;
  my $ArrayID = shift;
  my $rowID = shift;
  my $eccOption = shift;
  my $x = "PSoC3_ProgramRowFromHex ".$ArrayID." ".$rowID." ".$eccOption;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub PSoC3_ProgramRow {
  my $self = shift;
  my $ArrayID = shift;
  my $rowID = shift;
  my @data = @_;
  my $d = sprintf("0x%x 0x%x",$ArrayID,$rowID);
  foreach my $i (@data) {
    $d .= sprintf(" 0x%x",$i);
  }
  my $x = "PSoC3_WriteRow ". $d;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}


sub PSoC3_ReadRow {
  my $self = shift;
  my $ArrayID = shift;
  my $rowID = shift;
  my $eccOption = shift;
  @start_time = gettimeofday();
  my $l = "PSoC3_ReadRow " . $ArrayID ." ". $rowID ." ". $eccOption;
  my $x = $self->runcommand("PSoC3_ReadRow " . $ArrayID ." ". $rowID ." ". $eccOption);
  @end_time = gettimeofday();calc_delta();
  my @sout = split(" ",$x);
  pop(@sout); # pop off the status which should be 0
  my @out;
  foreach my $o (@sout) {
    push(@out,hex("0x".$o));
  }
  return $x;
}

sub PSoC3_EraseSector {
  my $self = shift;
  my $ArrayID = shift;
  my $SectorID = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("PSoC3_EraseSector" . " " . $ArrayID . " " . $SectorID);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub PSoC3_EraseRow {
  my $self = shift;
  my $ArrayID = shift;
  my $RowID = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("PSoC3_EraseRow" . " " . $ArrayID . " " . $RowID);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub PSoC3_WriteIO {
  my $self = shift;
  my $baseAddr = shift;
  my $data = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("PSoC3_WriteIO" . " " . $baseAddr . " " . $data);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub PSoC3_WriteNvlArray {
  my $self = shift;
  my $arrayID = shift;
  my $data = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("PSoC3_WriteNvlArray" . " " . $arrayID . " " . $data);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub PSoC3_Acquire {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("PSoC3_Acquire");
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub GetPower2 {
  my $self = shift;
  my $x = "GetPower2";
  my $o = $self->runcommand($x);
  my @arr = split(/\n/,$o);
  @end_time = gettimeofday();calc_delta();
  return @arr;
}
sub runfile {
  my $self = shift;
  @start_time = gettimeofday();
  my $path = shift;
  my $x = "runfile " . $path;
  my $o = $self->runcommand($x);
  @end_time = gettimeofday();calc_delta();
  return $o;
}
sub GetProgrammerVersion{
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("GetProgrammerVersion");
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub doread {
  my $self = shift;
  $out = "";
  while ($DataQueue->pending > 0){
      $out .=$DataQueue->dequeue;
  }
  #  print $out;
  $self->{out} = $out;
  return $out;
}
sub runcommand {
  my $self = shift;
  my $command = shift;
  #  printf ("running command %s\n",$command);
  $command =~ s/[\n]*$/\n\n\n/;
  return if(!(defined($WRFH) && defined($RDFH)));
  print $WRFH $command;
  # wait for a reply
  $self->{wait}=0;
  my $sleeptime = 100000;       # 100ms;
  my $timeoutus = $self->{timeout} * 1000000;
  my $o;
  while ($self->{wait} < $timeoutus) {
    usleep($sleeptime);
    $self->{wait}+=$sleeptime;
    my $u = $self->doread();
    $o .= $u;
    last if ($u =~ /OK$/);
  }
  $self->{result} = "fail";
  $o .= $self->doread();
  $self->{commandout} = $o;
  if ($o =~ /OK/) {
    $self->{result} = "pass";
  }
  if ($o =~ />>E/) {
    $self->{result} = "fail";
  }
  $o =~ s/^>.*\n*<//m;
  my $x = $o;

  my @stuff;
  @stuff = split(/\n/,$o);
  my $ok = pop(@stuff);
  $o = join("\n",@stuff);
  #  printf ("command %s %s %d returned %s\n",$command,$self->{result},$self->{wait},$o);
  return $o;
}
sub GetSiliconId {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("GetSiliconId");
  @end_time = gettimeofday();calc_delta();
  my @sid = split(" ",$x);
  return $x;
}
sub SetSiliconId {
  my $self = shift;
  @start_time = gettimeofday();
  my @sid = @_;
  my $sid = join " ",@sid;
  my $x = $self->runcommand("SetSiliconId ".$sid);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub Acquire {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("AcquireChip");
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub Calibrate{
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("Calibrate");
  @end_time = gettimeofday();calc_delta();
  return $x;

}
sub Erase {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("Erase");
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub Program {
  my $self = shift;
  @start_time = gettimeofday();
  my $savetimeout = $self->{timeout};
  $self->{timeout} = 200; # 20 seconds
  my $x = $self->runcommand("Program");
  @end_time = gettimeofday();calc_delta();
  $self->{timeout} = $savetimeout;
  return $x;
}
sub EraseBlock{
  my $self = shift;
  @start_time = gettimeofday();
  my $block = shift;
  my $bid = sprintf ("0x%x",$block);
  my $x = $self->runcommand("EraseBlock ". $bid);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub ReadBlock {
  my $self = shift;
  @start_time = gettimeofday();
  my $block = shift;
  my $bid = sprintf ("0x%x",$block);
  my $x = $self->runcommand("ReadBlock ". $bid);
  @end_time = gettimeofday();calc_delta();
  my @sout = split(" ",$x);
  pop(@sout); # pop off the status which should be 0
  my @out;
  foreach my $o (@sout) {
    push(@out,hex("0x".$o));
  }
  return @out;
}
sub ProgramBlock {
  my $self = shift;
  @start_time = gettimeofday();
  my $block = shift;
  my @data = @_;
  my $d = sprintf(" 0x%x",$block);
  foreach my $i (@data) {
    $d .= sprintf(" 0x%x",$i);
  }
  my $x = $self->runcommand("WriteBlock".$d);
  @end_time = gettimeofday();calc_delta();
  return $x;
}

sub ReleaseChip {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("ReleaseChip");
  @end_time = gettimeofday();calc_delta();
  return $x;
}

sub SetAcquireMode {
  my $self = shift;
  @start_time = gettimeofday();
  my $mode = shift;
  my $x = $self->runcommand("SetAcquireMode " . $mode);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub SetAutoReset {
  my $self = shift;
  @start_time = gettimeofday();
  my $mode = shift;
  my $x = $self->runcommand("SetAutoReset " . $mode);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub SetChipType {
  my $self = shift;
  @start_time = gettimeofday();
  my $type = shift;
  my $x = $self->runcommand("SetChipType " .$type);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub Verify {
  my $self = shift;
  my $savetimeout = $self->{timeout};
  $self->{timeout} = 200; # 20 seconds
  @start_time = gettimeofday();
  my $x = $self->runcommand("Verify");
  @end_time = gettimeofday();calc_delta();
  $self->{timeout} = $savetimeout;
  return $x;
  $self->{command} = shift;
}
sub PowerOn {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("PowerOn");
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub PowerOff {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("PowerOff");
  @end_time = gettimeofday();calc_delta();
  return $x;
}
# sub ReadProgram {
#   my $self = shift;
#   my $start = shift;
#   my $count = shift;
#   my $protref = shift;
#   my $st = sprintf(" 0x%x",$start);
#   my $cnt = sprintf(" 0x%x",$count);
#   @start_time = gettimeofday();
#   my $x = $self->runcommand("ReadProgram".$st.$cnt);
#   @end_time = gettimeofday();
#   my $dt = calc_delta();
#   printf("ReadProgram".$st.$cnt." %g\n",$dt);
#   # returns $cnt * 64 bytes of data in xxxx form + data in xx form
#   if($x =~ />E/) {
#     return -1;
#   }
#   my @out = split(" ",$x);
#   my @outdata;
#   my @outprot;
#   while (@out && $out[0]=~ /..(..)/) {
#     push @outdata,hex("0x".$1);
#     shift(@out);
#   }
#   while (@out && $out[0]=~ /(..)/) {
#     push @outprot,hex("0x".$1);
#     shift(@out);
#   }
#   if (defined ($protref)) {
#     @${protref} = @outprot;
#   }
#   return @outdata;
# }
sub ReadProm {
  my $self = shift;
  my @mstart_time = gettimeofday();
  my $startaddr = shift;
  my $count = shift;
  my $startblock = int($startaddr/64);
  my $blockcount = int(1+$count/64);
  my @d = $self->ReadProgram($startblock,$blockcount);
  #  will return a slice of size $count starting at
  #  $startaddr-$startblock*64
  my $startoff = $startaddr-$startblock*64;
  my $endoff = $startoff+$count;
  if ($endoff >= $#d) {
    $endoff = $#d;
  }
  ;
  @end_time = gettimeofday();
  @start_time = @mstart_time;
  calc_delta();
  return @d[$startoff ... $endoff];
}
sub HEX_ReadFile {
  my $self = shift;
  @start_time = gettimeofday();
  my $file = shift;
  my $x = $self->runcommand("HEX_ReadFile \"". $file . "\"");
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub HEX_WriteFile {
  my $self = shift;
  @start_time = gettimeofday();
  my $file = shift;
  my $x = $self->runcommand("HEX_WriteFile \"" . $file . "\"");
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub ReadHexData{
  my $self = shift;
  @start_time = gettimeofday();
  my $addr = shift;
  my $len = shift;
  my $command = sprintf("ReadHexData 0x%x 0x%x",$addr,$len);
  my $sout = $self->runcommand($command);
  my @sout = split(/ /,$sout);
  @end_time = gettimeofday();calc_delta();
  return @sout;
}
sub WriteHexData{
  my $self = shift;
  @start_time = gettimeofday();
  my $addr = shift;
  my $command = sprintf("WriteHexData 0x%x ",$addr);
  my $value;
  while (defined($value = shift)) {
    $command .= sprintf(" 0x%x",$value);
  }
  my $x = $self->runcommand($command);
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub ReadHexChecksum {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("ReadHexChecksum");
  @end_time = gettimeofday();calc_delta();
  return $x;
}
sub CheckSum {
  my $self = shift;
  @start_time = gettimeofday();
  my $blocks = shift;
  my $cmd = sprintf ("CheckSum 0x%x",$blocks);
  my $status = $self->runcommand($cmd);
  @end_time = gettimeofday();calc_delta();
  return $status;
}
sub SetPowerVoltage {
  my $self = shift;
  @start_time = gettimeofday();
  my $arg = shift;
  my $status = $self->runcommand("SetPowerVoltage $arg");
  @end_time = gettimeofday();calc_delta();
  return $status;
}
sub Protect {
  my $self = shift;
  @start_time = gettimeofday();
  my $status = $self->runcommand("Protect");
  @end_time = gettimeofday();calc_delta();
  return $status;
}
sub quit {
  my $self = shift;
  @start_time = gettimeofday();
  my $x = $self->runcommand("quit");
  @end_time = gettimeofday();calc_delta();
  # close the port
  close($WRFH)if (defined ($WRFH));
  close($RDFH)if (defined ($RDFH));
  close($ERFH)if (defined ($ERFH));
  undef($WRFH);
  undef($RDFH);
  undef($ERFH);
  waitpid($self->{pid},0);
  return $x;
}
sub AcquireAndCal {
  my $self = shift;
  my $a = $self->Acquire();
  my @nstart_time = @start_time;
  $self->Calibrate();
  @start_time = @nstart_time;
  calc_delta();
  return $a;
}
sub calc_delta {
  my $dsec = $end_time[0] - $start_time[0];
  my $dusec= $end_time[1] - $start_time[1];
  $dusec /= 1.0e6;
  $delta_time = $dsec + $dusec;
#   printf ("calc_delta %d.%d %d.%d %d.%g %g\n",
#           $start_time[0],$start_time[1],
#           $end_time[0],$end_time[1],
#           $dsec,$dusec,$delta_time);
  return $delta_time;
}
sub getfuntime {
  return $delta_time;
}
sub getretstatus {
  my $self = shift;
  my ($t,$s,$ok) = $self->{commandout} =~ /([0-9.]+) ([0-9a-fA-F]+) (.*)/;
  return hex("0x".$s);
}
sub getrettime {
  my $self = shift;
  my ($t,$s,$ok) = $self->{commandout} =~ /([-+e0-9.]+) ([0-9a-fA-F]+) (.*)/;
  $t ++;
  $t --;
  return $t;
}
use vars qw($AUTOLOAD);
sub AUTOLOAD {
  my $self = shift;
  my $retval;
  my $fun = $AUTOLOAD;
  $fun  =~ s/.*:://o;
  # look for fun in help
  my $x;
  my $fargs;
  ($x,$fargs) = $self->{help} =~ m/\n(${fun})\( *(.*) *\)\n/m;
  my @inargs;
  my @outargs;

  if(defined($fargs)) {
    my @fargs = split(",",$fargs);
    my $arg;
    foreach $arg(@fargs) {
      if($arg=~/IN/) {
        push(@inargs,$arg);
      }
      if($arg=~/OUT/) {
        push(@outargs,$arg);
      }
    }
  }
  if(defined($x)){
    my $d="";;
    foreach my $i (@_) {
      $d .= sprintf(" 0x%x",$i);
    }
    my $o = $self->runcommand($fun . $d);
    # convert out arguments
    # printf("AUTOLOAD $fun %s %s\n",join(",",@inargs),join(",",@outargs));
    # split output on newlines
    my @o = split(/\n/,$o);
    my $outputnumber = 0;
    foreach my $output (@o) {
      # convert.
      my $s = convert($outargs[$outputnumber],$output);
      push(@{$self->{outputs}},$s);
    }
    if($#o > 0) {
      return @{$self->{outputs}};
    }else {
      return $o;
    }
  }
  1;
}
my $ffun;
sub convert {
  my $thing = shift;
  if (!$thing ) {
    $out = "";
  }else{
    my @thing = split(/ */,$thing);
  my @in = @_;
  my $out;
  if($thing =~ /IN/) {
  }elsif($thing =~ /OUT/) {
    $ffun = $thing[0]."OUT";
    $out = eval "$ffun(\"$in[0]\");";
  }
  }
};
sub nvectorOUT {
  my $in = shift;
  my $out;
  my @inv = split("  *",$in);
  foreach my $inv (@inv) {
    my $hv = hex("0x".$inv);
    push(@{$out},$hv);
  }
  return $out;
}
1;